import { useEditor } from "@/modules/editor";
import { DesignComp } from "@/modules/editor/objects/DesignTemp/DesignComp";
import FormUI, { ColumnItem } from "@queenjs/components/FormUI";
import { defineComponent } from "vue";
import { any } from "vue-types";
import { createColorOpts } from "../../defines/formOpts/createColorOpts";
import { PageMusic } from "./PageMusic";
import { Select } from "ant-design-vue";
import { MusicOptions } from "./localMusic";
import { css } from "@linaria/core";
const styleColumns = (muisc?: string): ColumnItem[] => {
  return [
    {
      label: "背景颜色",
      dataIndex: "layout.background.color",
      ...createColorOpts(),
    },
    {
      label: "背景音乐",
      dataIndex: "value.music",
      component: Select,
      props: {
        class: "w-full flex-1 overflow-hidden",
        value: muisc,
        options: [{ label: "无", value: "" }, ...MusicOptions],
      },
    },{
      label: "页面模式",
      dataIndex: "value.pageMode",
      component: Select,
      props: {
        class: "w-full flex-1 overflow-hidden",
        options: [{ label: "长页", value: "long" }, { label: "翻页", value: "short" }],
      },
    },
    {
      dataIndex: "value.music",
      component: PageMusic,
      isVisible: (value, data) => data?.value?.music != "" && muisc != "",
    },
  ];
};
export const PageForm = defineComponent({
  props: {
    component: any<DesignComp>().isRequired,
  },
  setup(props) {
    const { actions, helper } = useEditor();
    function changeVal(e: { dataIndex: string; value: any }) {
      actions.updateCompData(props.component, e.dataIndex, e.value);
    }

    return () => {
      const compMusic = helper.findRootComp()?.value.music || "";
      const curValue = MusicOptions.find((e) => {
        return e.value == compMusic;
      });

      return (
        <div class={formStyle}>
          <div class="text-white">页面样式</div>
          <FormUI
            data={props.component}
            columns={styleColumns(curValue?.value || "")}
            onChange={changeVal}
          />
        </div>
      );
    };
  },
});

const formStyle = css`
  .item_label {
    color: #A9ABAF !important;
  }
`