import { Dict_Imgs } from "@/dict";
import { createAttrsForm } from "../../defines/createAttrsForm";
import { createCompHooks } from "../../defines/createCompHooks";
import { InputNumber, Switch } from "ant-design-vue";
import { createColorOpts } from "../../defines/formOpts/createColorOpts";
import Slider from "../../formItems/Slider";

export { Component } from "./component";

export const options = {
  name: "正多边形",
  thumbnail: require("@/modules/editor/assets/icons/polygon.svg"),
};

export const { createComp, useCompData } = createCompHooks({
  value: {
    lineColor: "#000000",
    lineWidth: 0,
    isFill: true,
    fillColor: "#A4A4A4",
    points: [],
    isClose: true,
    reverseFill: false,
    edges: 6,
    baseAngle: 0,
  },
  layout: {
    size: [400, 400],
  },
});

export const Form = createAttrsForm([

    {
        label: "边数",
        dataIndex: "value.edges",
        component: Slider,
        props: {
            defaultValue: 0,
            min: 3,
            max: 36,
            step: 1,
          },
    },
    {
        label: "角度",
        dataIndex: "value.baseAngle",
        component: Slider,
        props: {
            defaultValue: 0,
            min: 0,
            max: 180,
            step: 1,
        },
    },

  {
    label: "线宽",
    dataIndex: "value.lineWidth",
    component: InputNumber,
  },
  {
    label:"颜色",
    dataIndex: "value.lineColor",
    ...createColorOpts(),
  },
  {
    label: "是否填充",
    dataIndex: "value.isFill",
    component: "Switch",
  },
  {
    label:"填充颜色",
    dataIndex: "value.fillColor",
    ...createColorOpts(),
    isVisible: (value, data) => data?.value?.isFill == true,
  },
  {
    label: "反向填充",
    dataIndex: "value.reverseFill",
    component: "Switch",
    isVisible: (value, data) => data?.value?.isFill == true,
  },
  
]);
