import Command from "@ckeditor/ckeditor5-core/src/command";
import { Letter_Spacing } from "./utils";

export default class LetterSpacingCommand extends Command {
  constructor(editor: any) {
    super(editor);
  }
  refresh() {
    const model = this.editor.model;
    const doc = model.document;
    this.value = doc.selection.getAttribute(Letter_Spacing) || "0";
    this.isEnabled = model.schema.checkAttributeInSelection(
      doc.selection,
      Letter_Spacing
    );
  }

  execute(options = {} as any) {
    const model = this.editor.model;
    const document = model.document;
    const selection = document.selection;
    const value = options.value;
    const batch = options.batch;

    const updateAttribute = (writer: any) => {
      if (selection.isCollapsed) {
        return;
      }
      const ranges = model.schema.getValidRanges(
        selection.getRanges(),
        Letter_Spacing
      );
      for (const range of ranges) {
        if (value) {
          writer.setAttribute(Letter_Spacing, value, range);
        } else {
          writer.removeAttribute(Letter_Spacing, range);
        }
      }
    };
    if (batch) {
      model.enqueueChange(batch, (writer) => {
        updateAttribute(writer);
      });
    } else {
      model.change((writer) => {
        updateAttribute(writer);
      });
    }
  }
}
