import Plugin from "@ckeditor/ckeditor5-core/src/plugin";
import HeadlessEditor from "../EditorCustom";
import LetterSpacingCommand from "./LetterSpacingCommand";
import { Letter_Spacing, buildDefinition, normalizeOptions } from "./utils";

export default class LetterSpacingEditing extends Plugin {
  constructor(editor: HeadlessEditor) {
    super(editor);

    editor.config.define("letterSpacing", {
      options: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
      supportAllValues: false,
    });
  }

  /**
   * @inheritDoc
   */
  init() {
    const editor = this.editor;
    editor.model.schema.extend("$text", {
      allowAttributes: Letter_Spacing,
    });
    editor.model.schema.setAttributeProperties(Letter_Spacing, {
      isFormatting: true,
      copyOnEnter: true,
    });

    const supportAllValues = editor.config.get(
      "letterSpacing.supportAllValues"
    );
    const options = normalizeOptions(
      this.editor.config.get("letterSpacing.options")
    ).filter((item: any) => item.model);
    const definition = buildDefinition(options);

    if (supportAllValues) {
      this._prepareAnyValueConverters();
    } else {
      editor.conversion.attributeToElement(definition);
    }

    editor.commands.add("letterSpacing", new LetterSpacingCommand(editor));
  }
  _prepareAnyValueConverters() {
    const editor = this.editor;
    editor.conversion.for("downcast").attributeToElement({
      model: Letter_Spacing,
      view: (attributeValue, { writer }) => {
        if (!attributeValue) {
          return;
        }
        return writer.createAttributeElement(
          "span",
          { style: "letter-spacing:" + attributeValue },
          { priority: 7 }
        );
      },
    });
    editor.conversion.for("upcast").elementToAttribute({
      model: {
        key: Letter_Spacing,
        value: (viewElement: any) => viewElement.getStyle("letter-spacing"),
      },
      view: {
        name: "span",
        styles: {
          "letter-spacing": /.*/,
        },
      },
    });
  }
}
