import Command from "@ckeditor/ckeditor5-core/src/command";
import { Text_Stroke } from "./utils";

export default class TextStrokeCommand extends Command {
  constructor(editor: any) {
    super(editor);
  }
  refresh() {
    const model = this.editor.model;
    const doc = model.document;
    this.value = doc.selection.getAttribute(Text_Stroke);
    this.isEnabled = model.schema.checkAttributeInSelection(
      doc.selection,
      Text_Stroke
    );
  }

  execute(options = {} as any) {
    const model = this.editor.model;
    const document = model.document;
    const selection = document.selection;
    const value = options.value;
    const batch = options.batch;

    const updateAttribute = (writer: any) => {
      if (selection.isCollapsed) {
        return;
      }
      const ranges = model.schema.getValidRanges(
        selection.getRanges(),
        Text_Stroke
      );
      for (const range of ranges) {
        if (value) {
          writer.setAttribute(Text_Stroke, value, range);
        } else {
          writer.removeAttribute(Text_Stroke, range);
        }
      }
    };
    if (batch) {
      model.enqueueChange(batch, (writer) => {
        updateAttribute(writer);
      });
    } else {
      model.change((writer) => {
        updateAttribute(writer);
      });
    }
  }
}
