import Plugin from "@ckeditor/ckeditor5-core/src/plugin";
import HeadlessEditor from "../EditorCustom";
import LetterSpacingCommand from "./TextStrokeCommand";
import { Text_Stroke } from "./utils";

export default class TextStrokeEditing extends Plugin {
  constructor(editor: HeadlessEditor) {
    super(editor);

    // editor.config.define("letterSpacing", {
    //   options: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
    //   supportAllValues: false,
    // });
  }

  /**
   * @inheritDoc
   */
  init() {
    const editor = this.editor;
    editor.model.schema.extend("$text", {
      allowAttributes: Text_Stroke,
    });
    editor.model.schema.setAttributeProperties(Text_Stroke, {
      isFormatting: true,
      copyOnEnter: true,
    });

    // const options = normalizeOptions(
    //   this.editor.config.get("letterSpacing.options")
    // ).filter((item: any) => item.model);

    this._prepareAnyValueConverters();

    editor.commands.add("textStroke", new LetterSpacingCommand(editor));
  }
  _prepareAnyValueConverters() {
    const editor = this.editor;
    editor.conversion.for("downcast").attributeToElement({
      model: Text_Stroke,
      view: (attributeValue, { writer }) => {
        if (!attributeValue) {
          return;
        }
        return writer.createAttributeElement(
          "span",
          { style: "-webkit-text-stroke:" + attributeValue },
          { priority: 8 }
        );
      },
    });
    editor.conversion.for("upcast").elementToAttribute({
      model: {
        key: Text_Stroke,
        value: (viewElement: any) =>
          viewElement.getStyle("-webkit-text-stroke"),
      },
      view: {
        name: "span",
        styles: {
          "-webkit-text-stroke": /.*/,
        },
      },
    });
  }
}
