export const Letter_Spacing = "letterSpacing";
export const Text_Stroke = "textStroke";

export type optionsKey = Record<string, any>;

export function buildDefinition(options: any) {
  const definition = {
    model: {
      key: "letterSpacing",
      values: [] as any,
    },
    view: {} as optionsKey,
    upcastAlso: {} as optionsKey,
  };
  for (const option of options) {
    definition.model.values.push(option.model);
    definition.view[option.model] = option.view;
    if (option.upcastAlso) {
      definition.upcastAlso[option.model] = option.upcastAlso;
    }
  }
  return definition;
}
export function normalizeOptions(configuredOptions: any) {
  return configuredOptions
    .map((item: any) => optionDefinition(item))
    .filter((option: any) => option !== undefined);
}

function optionDefinition(option: any) {
  if (typeof option === "object") {
    return option;
  }

  if (option === "default") {
    return {
      model: undefined,
      title: "Default",
    };
  }

  const sizePreset = parseFloat(option);

  if (isNaN(sizePreset)) {
    return;
  }

  return generatePixelPreset(sizePreset);
}

function generatePixelPreset(size: number) {
  const sizeName = String(size);
  return {
    title: sizeName,
    model: `${size}px`,
    view: {
      name: "span",
      styles: {
        "letter-spacing": `${size}px`,
      },
      priority: 5,
    },
  };
}
