import { defineComponent, ref, watch, onMounted,nextTick } from "vue";
import { string } from "vue-types";
import { useCompData } from ".";
import { View } from "../View";
import { useEditor } from "@/modules/editor";
import { IconPicture } from "@queenjs/icons";
import { css } from "@linaria/core";

export const Component = defineComponent({
  props: {
    compId: string().isRequired,
  },
  setup(props) {
    const { store, controls, actions, helper } = useEditor();
    const comp = useCompData(props.compId);
    const { value } = comp;
    const videoRef = ref<HTMLVideoElement>();

    async function changeVal() {
      try {
        const url = await controls.pickCtrl.pickOneImage();
        if (!url) return;
        value.url = url;
      } catch (error) {
        console.log(error);
      }
    }

    function updateSize() {
      const { videoWidth, videoHeight } = videoRef.value as HTMLVideoElement;
      if (videoHeight == 0 || videoWidth == 0) return;
      const t = value.ratio || videoWidth * 1.0 / videoHeight;
      comp.setH(comp.getW() / t);
      actions.onCompLayoutUpdated(comp);
      helper.extendStreamCard( store.currStreamCardId );
    }

    watch(
      () => [value.ratio],
      () => {
        updateSize();
      }
    );
    onMounted(() => {
      const el = videoRef.value as HTMLVideoElement;

      if (store.isEditMode) {
        el.addEventListener("loadedmetadata", ()=>{
          const { videoWidth, videoHeight } = videoRef.value as HTMLVideoElement;
          if ( videoHeight != 0 && videoHeight != 0) {
              const r = videoWidth * 1.0 / videoHeight;
              if (comp.value.ratio == 0 && Math.abs( comp.layout.size[0] / comp.layout.size[1] - r) > 0.01) {
                 updateSize();
                 return;
              }
          } else {
            setTimeout(() => {
              const { videoWidth, videoHeight } = videoRef.value as HTMLVideoElement;
              const r = videoWidth * 1.0 / videoHeight;
    
              console.log("loadedmetadata==>",videoWidth,videoHeight,  comp.value.ratio, r, comp.layout.size);
    
              if (comp.value.ratio == 0 && Math.abs( comp.layout.size[0] / comp.layout.size[1] - r) > 0.01) {
                  updateSize();
                  console.log("updated sizeed");
              }
            }, 1000);
          }
        })
      }

      el.addEventListener("play", () => {
        controls.mediaCtrl.pauseOtherMedia(props.compId);
      });
      controls.mediaCtrl.setMediasInstance(props.compId, videoRef.value);
    });
    return () => {
      const options: any = {};
      if (value.autoplay) options.autoplay = false;
      if (value.loop) options.loop = true;
      if (value.controls) options.controls = true;
      if (value.poster) options.poster = value.poster;

      return (
        <View class={rootCls} compId={props.compId}>
          <video
            ref={videoRef}
            key={value.url}
            class="w-full object-cover"
            style={{ aspectRatio: value.ratio }}
            {...options}
          >
            <source src={value.url}></source>
          </video>
          {store.isEditMode && (
            <IconPicture class={btnCls} onClick={changeVal} />
          )}
        </View>
      );
    };
  },
});

const btnCls = css`
  display: none;
  position: absolute;
  top: 50%;
  left: 50%;
  padding: 20px;
  font-size: 37.5px;
  color: #666;
  border-radius: 50%;
  background-color: rgba(255, 255, 255, 0.7);
  transform: translate(-50%, -50%);
  @apply shadow;
`;

const rootCls = css`
  &:hover {
    .${btnCls} {
      display: block;
    }
  }
`;
