// import AMapLoader from "@amap/amap-jsapi-loader";
import { Input } from "ant-design-vue";
import { defineComponent, onMounted, reactive, ref, watch } from "vue";
import { any } from "vue-types";
declare const AMap: any;

export default defineComponent({
  props: {
    value: any(),
  },
  emits: ["change"],
  setup(props, { emit }) {
    // const AMap = ref();
    const map = ref();
    const marker = ref();
    const geocoder = ref();
    const autoComplete = ref();

    const state = reactive({
      address: "天安门",
      tips: [],
      lnglat: [116.397428, 39.90923],
      visible: false,
      ...props.value,
    });

    function clickMap(e: any) {
      state.lnglat = [e.lnglat.getLng(), e.lnglat.getLat()];
      updateLnglat();
      regeoCode();
    }

    function updateLnglat() {
      marker.value?.setPosition(state.lnglat);
      map.value.add(marker.value);
      map.value.setZoomAndCenter(11, state.lnglat);
      emit("change", { address: state.address, lnglat: state.lnglat });
    }

    function geoCode() {
      geocoder.value.getLocation(
        state.address,
        (status: string, result: any) => {
          if (status === "complete" && result.geocodes.length) {
            const lnglat = result.geocodes[0].location;
            state.lnglat = lnglat;
            updateLnglat();
          } else {
            console.error("根据地址查询位置失败");
          }
        }
      );
    }

    function regeoCode() {
      geocoder.value.getAddress(
        state.lnglat,
        function (status: any, result: any) {
          if (status === "complete" && result.regeocode) {
            const address = result.regeocode.formattedAddress;
            state.address = address;
          } else {
            console.error("根据经纬度查询地址失败");
          }
        }
      );
    }

    const searchAddress = () => {
      autoComplete.value.search(
        state.address,
        function (status: any, result: any) {
          if (status === "complete") {
            // 搜索成功时，result即是对应的匹配数据
            state.tips = result.tips || [];
            state.visible = true;
          } else {
            console.error("输入提示查询失败");
            geoCode();
          }
        }
      );
    };

    const initMap = () => {
      // AMapLoader.load({
      //   key: "389e81c4c61fd6dff4ab8f742af82e5f", // 申请好的Web端开发者Key，首次调用 load 时必填
      //   version: "2.0", // 指定要加载的 JSAPI 的版本，缺省时默认为 1.4.15
      //   plugins: ["AMap.Scale", "AMap.ToolBar", "AMap.MouseTool"], // 需要使用的的插件列表，如比例尺'AMap.Scale'等
      //   Loca: {
      //     version: "2.0.0",
      //   },
      // })
      //   .then((res) => {
      //     AMap.value = res;
      map.value = new AMap.Map("mapJsContainer", {
        // viewMode: "3D", //是否为3D地图模式
        zoom: 11, //初始化地图级别
        center: state.lnglat, //初始化地图中心点位置
      });

      AMap.plugin(
        ["AMap.Geocoder", "AMap.AutoComplete", "AMap.ToolBar"],
        () => {
          // 使用geocoder做地理/逆地理编码
          geocoder.value = new AMap.Geocoder();
          autoComplete.value = new AMap.AutoComplete({
            city: "全国",
          });

          const toolbar = new AMap.ToolBar({ position: "LT" }); // 缩放工具条实例化
          map.value.addControl(toolbar);
        }
      );

      marker.value = new AMap.Marker({
        position: state.lnglat, //位置
      });
      map.value.add(marker.value); //添加到地图

      map.value.on("click", clickMap);
      // })
      // .catch((e) => {
      //   console.log("error", e);
      // });
    };

    function selectAddress(tip: any) {
      state.address = tip.name;
      state.lnglat = [tip.location?.lng, tip.location?.lat];
      state.visible = false;
      updateLnglat();
    }

    onMounted(() => {
      initMap();
    });

    watch(
      () => [props.value],
      () => {
        const { lnglat } = props.value;
        if (state.lnglat[0] == lnglat[0] && state.lnglat[1] == lnglat[1])
          return;
        state.address = props.value.address;
        state.lnglat = props.value.lnglat;
        updateLnglat();
      }
    );

    return () => {
      return (
        <div class="w-1/1 relative">
          <Input.Search
            value={state.address}
            placeholder="请输入地名"
            class="w-1/1"
            // onSearch={searchAddress}
            onChange={(e) => {
              state.address = e.target.value || "";
              searchAddress();
            }}
          />
          {state.visible && (
            <div class="absolute left-0 right-0 top-38px select_card z-999 text-14px rounded-2px bg-light-50">
              {state.tips?.map((tip: any, index: number) => (
                <div
                  key={index}
                  class="flex px-15px py-8px overflow-hidden cursor-pointer hover:(opacity-80) transition"
                  onClick={() => selectAddress(tip)}
                >
                  <div class="text-dark-100 truncate">{tip.name}</div>
                  <div class="flex-1 ml-4px truncate text-gray">
                    {tip.district}
                    {tip.address}
                  </div>
                </div>
              ))}
            </div>
          )}
          <div
            id="mapJsContainer"
            class="w-1/1 h-200px mt-10px rounded-2px"
          ></div>
        </div>
      );
    };
  },
});
