import { Divider } from "ant-design-vue";
import { defineComponent, onMounted, reactive } from "vue";
import { any, bool } from "vue-types";
import ColorList, { gradientColorType } from "./ColorList";

const pureColors = [
  "#111111",
  "#545454",
  "#737373",
  "#A6A6A6",
  "#FFFFFF",
  "#EC635D",
  "#F5BF6A",
  "#93D668",
  "#5970F6",
  "#8354F5",
];

const gradientColors: gradientColorType[] = [
  {
    deg: 90,
    type: "linear",
    colors: ["#212121", "#656565"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#AEAEAE", "#F0F0F0"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#FCEFBB", "#F4B8EA"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#CCF1E0", "#A5C3F4"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#EB995A", "#F8D86E"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#EB4F40", "#EF8E58"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#85C6BD", "#F0DD75"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#8367F3", "#82CFE6"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#ED7FB1", "#F8D378"],
  },
  {
    deg: 90,
    type: "linear",
    colors: ["#935EDF", "#E69069"],
  },
];

export default defineComponent({
  props: {
    value: any<string | gradientColorType>().isRequired,
    showGradient: bool().def(true),
  },
  emits: ["change"],
  setup(props, { emit }) {
    const state = reactive({
      latestColors: [],
    }) as any;

    const selectColor = (data: any) => {
      emit("change", data);
    };

    onMounted(() => {
      const list = localStorage.getItem("latestColors");
      let colors = list ? JSON.parse(list) : ["#fff"];
      if (!props.showGradient) {
        colors = colors.filter((d: any) => typeof d == "string");
      }
      colors.splice(10);
      state.latestColors = colors;
    });

    return () => {
      const { showGradient } = props;
      return (
        <div class="w-280px px-5px py-10px rounded">
          <div class="flex items-center justify-between">
            <span class="text-16px">色彩库</span>
            {/* <IconClose class="text-18px" /> */}
          </div>
          <div class="mt-15px mb-10px">近期使用</div>
          <ColorList
            value={props.value}
            data={state.latestColors}
            onChange={(data) => emit("change", data)}
          />
          <Divider />
          <div class="mb-10px">纯色库</div>
          <ColorList
            value={props.value}
            data={pureColors}
            onChange={selectColor}
          />
          {showGradient && (
            <>
              <Divider />
              <div class="mb-10px">渐变</div>
              <ColorList
                value={props.value}
                data={gradientColors}
                onChange={selectColor}
              />
            </>
          )}
        </div>
      );
    };
  },
});
