import { css } from "@linaria/core";
import { IconAddLine } from "@queenjs/icons";
import { Divider, Popover, Tabs } from "ant-design-vue";
import { computed, defineComponent, onMounted, reactive, watch } from "vue";
import { any, bool } from "vue-types";
import ColorList, { ColorItem, gradientColorType } from "./ColorList";
import Picker from "./Picker";

export default defineComponent({
  props: {
    value: any<string | gradientColorType>().isRequired,
    showGradient: bool().def(true),
  },
  emits: ["change"],
  setup(props, { emit }) {
    const state = reactive({
      key: typeof props.value == "string" ? "0" : "1",
    });

    return () => {
      const { showGradient } = props;
      if (!showGradient) {
        return (
          <PureColor
            value={props.value}
            onChange={(data) => emit("change", data)}
          />
        );
      }
      return (
        <div class="w-280px px-5px pb-10px ">
          <Tabs
            centered
            destroyInactiveTabPane
            activeKey={state.key}
            class={tabStyles}
            onChange={(v: any) => (state.key = v)}
          >
            <Tabs.TabPane tab="纯色" key={"0"}>
              <PureColor
                value={props.value}
                onChange={(data) => emit("change", data)}
              />
            </Tabs.TabPane>
            <Tabs.TabPane tab="渐变" key={"1"}>
              <MutiColor
                value={props.value}
                onChange={(data) => emit("change", data)}
              />
            </Tabs.TabPane>
          </Tabs>
        </div>
      );
    };
  },
});

const PureColor = defineComponent({
  props: {
    value: any<string | gradientColorType>().isRequired,
  },
  emits: ["change"],
  setup(props, { emit }) {
    onMounted(() => {
      if (typeof props.value !== "string") {
        emit("change", props.value.colors[0]);
      }
    });
    return () => {
      const color =
        typeof props.value == "string" ? props.value : props.value.colors[0];
      return <Picker value={color} onChange={(data) => emit("change", data)} />;
    };
  },
});

const MutiColor = defineComponent({
  props: {
    value: any<string | gradientColorType>().isRequired,
  },
  emits: ["change"],
  setup(props, { emit }) {
    const state = reactive({
      colors: getColors(),
      gradientColors: computed(() => {
        const list: any = [
          { type: "linear", deg: 90 },
          { type: "linear", deg: 180 },
          { type: "linear", deg: 135 },
          { type: "radial" },
        ];
        return list.map((d: any) => ({ ...d, colors: state.colors }));
      }),
    });

    function getColors() {
      let colors: string[] = [];
      if (typeof props.value == "string") {
        colors = [props.value, "#fff"];
      } else {
        colors = props.value.colors;
      }
      return colors;
    }

    const deleteColor = (index: number) => {
      state.colors.splice(index, 1);
    };

    const addColor = () => {
      state.colors.push(state.colors[0]);
      setTimeout(() => {
        const elements = document.querySelectorAll(".jsColorItem");
        const element = elements[elements.length - 1];
        // @ts-ignore
        element.click();
      }, 0);
    };

    const changeColor = (value: any, index: number) => {
      state.colors[index] = value;
    };

    onMounted(() => {
      if (typeof props.value == "string") {
        emit("change", state.gradientColors[0]);
      }
    });

    watch(
      () => props.value,
      () => (state.colors = getColors())
    );

    return () => {
      return (
        <div>
          <div class="mb-10px">渐变色</div>
          <div class="grid gap-16px grid-cols-5 mr-5px">
            {state.colors.map((d, i) => {
              return (
                <Popover
                  key={i}
                  content={
                    <Picker
                      value={d}
                      onChange={(value) => changeColor(value, i)}
                    />
                  }
                  placement="bottomRight"
                  // visible={libVisible}
                  trigger="click"
                >
                  <ColorItem
                    class="jsColorItem"
                    color={d}
                    showDelete={state.colors.length > 2}
                    onDelete={() => deleteColor(i)}
                  />
                </Popover>
              );
            })}
            {state.colors.length <= 9 && (
              <div
                class="w-38px h-38px rounded cursor-pointer bg-[#3b3b3b] flex items-center justify-center transition hover:opacity-70"
                onClick={addColor}
              >
                <IconAddLine class="text-22px text-dark-300" />
              </div>
            )}
          </div>
          <Divider />
          <div class="mb-10px">风格</div>
          <ColorList
            value={props.value}
            data={state.gradientColors}
            columns={4}
            onChange={(data) => emit("change", data)}
          />
        </div>
      );
    };
  },
});

const tabStyles = css`
  .ant-tabs-nav-list {
    flex: 1;
  }
  .ant-tabs-tab {
    justify-content: center;
    flex: 1;
  }
  .ant-tabs-tab.ant-tabs-tab-active .ant-tabs-tab-btn {
    color: #fff;
  }
`;
