import { IconPicker } from "@/assets/icons";
import iro from "@jaames/iro";
import { css } from "@linaria/core";
import { Input } from "ant-design-vue";
import Color from "color";
import { nanoid } from "nanoid";
import { defineComponent, nextTick, onMounted, reactive, ref } from "vue";
import { string } from "vue-types";

declare global {
  interface Window {
    EyeDropper?: any;
  }
}

export default defineComponent({
  props: {
    value: string().def("#fff"),
  },
  emits: ["change"],
  setup(props, { emit }) {
    const containerId = `id_${nanoid(5)}`;
    const pickr = ref<any>(null);

    const state = reactive({
      showPicker: false,
    });

    function initPicker() {
      if (window.EyeDropper) state.showPicker = true;
      const element = document.querySelector(`.${containerId}`);
      if (!element) return;
      // @ts-ignore
      pickr.value = new iro.ColorPicker(element, {
        width: 270,
        boxHeight: 150,
        color: props.value,
        sliderSize: 16,
        layout: [
          {
            component: iro.ui.Box,
          },
          {
            component: iro.ui.Slider,
            options: {
              sliderType: "hue",
              id: "hue-slider",
              handleSvg: "#handle",
              handleProps: { x: -3, y: -3 },
              handleRadius: 5,
            },
          },
          {
            component: iro.ui.Slider,
            options: {
              sliderType: "alpha",
              handleSvg: "#handle",
              handleProps: { x: -3, y: -3 },
              handleRadius: 5,
            },
          },
        ],
      });

      pickr.value.on("color:change", function (color: any) {
        if (color.alpha == 1) {
          emit("change", color.hexString);
          return;
        }
        emit("change", color.hex8String);
      });
    }

    onMounted(() => {
      nextTick(() => {
        initPicker();
      });
    });

    function pickColor() {
      if (!window.EyeDropper) {
        return;
      }
      const eyeDropper = new window.EyeDropper();
      eyeDropper
        .open()
        .then((result: any) => {
          emit("change", result);
        })
        .catch((e: any) => {
          console.log("e: ", e);
        });
    }

    return () => {
      const colorObj = Color(props.value);
      const opacity = (colorObj.alpha() * 100).toFixed(0) + "%";

      return (
        <div class={styles}>
          <div class={containerId}></div>
          <div class="mt-15px flex items-center justify-between">
            <Input
              readonly
              value={colorObj.hex()}
              class="flex-1 bg-dark-300 h-36px hover:bg-dark-300 text-center"
              bordered={false}
            />
            <Input
              readonly
              value={opacity}
              class="w-60px ml-10px bg-dark-300 h-36px hover:bg-dark-300 text-center"
              bordered={false}
            />
            {state.showPicker && (
              <IconPicker
                class="ml-10px text-36px bg-dark-300 rounded-2px transition cursor-pointer !hidden hover:opacity-60"
                onClick={pickColor}
              />
            )}
          </div>
          <svg style="display:none;">
            <circle
              fill="#fff"
              id="handle"
              cx="8"
              cy="8"
              r="6"
              // stroke-width="1"
              // stroke="#fff"
            ></circle>
          </svg>
        </div>
      );
    };
  },
});

const styles = css`
  .IroBox {
    border-radius: 6px !important;
  }
`;
