import { useEditor } from "@/modules/editor";
import { ICompKeys } from "@/modules/editor/typings";
import { useReactive } from "@queenjs/use";
import { defineComponent } from "vue";
import { Container, Draggable } from "vue-dndrop";

export default defineComponent({
  setup() {
    const editor = useEditor();
    const { compUICtrl } = editor.controls;

    const state = useReactive(() => ({
      basicComps() {
        return ["Map"].map(
          (key) => compUICtrl.state.components.get(key) as any
        );
      },
    }));

    return () => {
      return (
        <div class="flex-1 overflow-x-hidden overflow-y-auto scrollbar">
          <Container
            class="grid grid-cols-3 gap-10px"
            behaviour="copy"
            group-name="canvas"
            get-child-payload={(index: number) => {
              return state.basicComps[index].compKey;
            }}
          >
            {state.basicComps.map((item) => {
              return (
                <Draggable key={item.compKey}>
                  <div
                    class="draggable-item py-10px bg-dark-200 rounded text-center cursor-pointer hover:bg-dark-300 transition"
                    onClick={() =>
                      editor.actions.clickCompToDesign(
                        item.compKey as ICompKeys
                      )
                    }
                  >
                    <img class="h-28px m-6px" src={item.thumbnail} />
                    <div class="text-center mt-5px">{item.name}</div>
                  </div>
                </Draggable>
              );
            })}
          </Container>
        </div>
      );
    };
  },
});
