import { cx } from "@linaria/core";
import { defineComponent } from "vue";
import { Container, Draggable } from "vue-dndrop";
import { array, bool, number, string } from "vue-types";

import { useEditor } from "@/modules/editor";
import { Image } from "@queenjs/ui";

export const CompList = defineComponent({
  props: {
    columns: number().def(1),
    dataSource: array<compType>().isRequired,
    gap: string().def("10px"),
    isSys: bool().isRequired,
  },

  setup(props) {
    const editor = useEditor();

    return () => {
      const { columns, dataSource, gap, isSys } = props;
      return (
        <Container
          class={cx("grid", `grid-cols-${columns}`, `gap-${gap}`)}
          behaviour="copy"
          group-name="canvas"
          animation-duration={0}
          get-child-payload={(index: number) => {
            return {
              type: "CompCard",
              data: { id: dataSource[index]._id, isSys },
            };
          }}
        >
          {dataSource.map((comp) => {
            return (
              <Draggable>
                <div
                  key={comp._id}
                  title={comp.title}
                  class="draggable-item h-1/1 bg-light-50 rounded-4px cursor-pointer transition overflow-hidden hover:(opacity-80)"
                >
                  <Image
                    class="w-full h-1/1 !object-contain"
                    src={comp.thumbnail}
                    onClick={() => {
                      editor.actions.clickCompUserToDesign(comp._id, isSys);
                    }}
                    size={480}
                  />
                </div>
              </Draggable>
            );
          })}
        </Container>
      );
    };
  },
});
