import Empty from "@/components/Empty";
import { useEditor } from "@/modules/editor";
import { useResource } from "@/modules/resource";
import { IconEyeOn } from "@queenjs/icons";
import { Image, Loadmore } from "@queenjs/ui";
import { defineUI, queenApi } from "queenjs";
import Modal from "queenjs/adapter/vue/components/modal";
import { Container, Draggable } from "vue-dndrop";
import PreviewTplModal from "./PreviewTplModal";
import { IconPreview } from "@/assets/icons";

export default defineUI({
  setup() {
    const editor = useEditor();
    const source = useResource();
    const ctrl = source.controls.sysTplListCtrl;
    ctrl.loadPage(1);

    const preview = (record: any) => {
      queenApi.dialog(
        <PreviewTplModal
          data={record}
          onClick={async () => {
            await editor.actions.clickTplToDesign(record);
            Modal.clear();
          }}
        ></PreviewTplModal>,
        {
          width: "1000px",
          title: "模板预览",
          destroyOnClose: true,
        }
      );
    };

    return () => {
      const dataSource = ctrl.state.list;

      return (
        <div>
          <Container
            class="space-y-10px"
            behaviour="copy"
            group-name="canvas"
            animation-duration={0}
            get-child-payload={(index: number) => {
              return {
                type: "tpl",
                data: dataSource[index],
              };
            }}
          >
            {dataSource.map((item: any) => {
              return (
                <Draggable key={item._id}>
                  <div
                    class="draggable-item relative"
                    key={item._id}
                    title={item.title}
                    style={{ aspectRatio: 1 }}
                    onClick={() => editor.actions.clickTplToDesign(item)}
                  >
                    <Image
                      class="w-full rounded pointer-events-none"
                      src={item.thumbnail}
                      size={240}
                    />
                    <div class="absolute w-1/1 h-1/1 top-0 left-0 bg-dark-900 bg-opacity-30 transition opacity-0 hover:(opacity-100) text-dark-500">
                      <IconPreview
                        class="absolute right-6px top-6px text-22px bg-light-50 rounded-2px cursor-pointer hover:opacity-80 text-dark-500 transition"
                        onClick={(e: any) => {
                          e.stopPropagation();
                          preview(item);
                        }}
                      />
                    </div>
                  </div>
                </Draggable>
              );
            })}
          </Container>
          {dataSource.length == 0 ? (
            <Empty class="pt-150px" />
          ) : (
            <Loadmore
              class="mt-20px"
              loading={ctrl.state.loading}
              canLoad={ctrl.state.canLoadNext}
              onChange={ctrl.loadNextPage}
            />
          )}
        </div>
      );
    };
  },
});
