import { ModuleControl } from "queenjs";
import { reactive } from "vue";
import { EditorModule } from "../../module";
import { CompObject } from "../SelectCtrl/compObj";
import { ObjsContainer } from "../SelectCtrl/ObjsContainer";
import { Matrix } from "../SelectCtrl/matrix";
import { TransformCtrl } from "../TransformCtrl";
import { CreateTransform } from "../../components/CompUI/basicUI/Transfer/transform";

function createImage(url:string) :Promise<HTMLImageElement> {
    return new Promise((r)=>{
        const img= new Image();
        img.onload = ()=>{
            r(img);
        }
        img.src = url;
    })
}

export class ImageCropperCtrl extends ModuleControl<EditorModule> {
    state = reactive({
        compId: "",
        visible: false,
    });

    modifyCtrl = new TransformCtrl();

    currSelected :string[]= [];
    initValue = {
        w: 0,
        h: 0,
        matrix: ""
    }
    lastValue = {
        w: 0,
        h: 0,
        matrix: ""
    }

    constructor(moduel:EditorModule) {
        super(moduel);

        this.modifyCtrl.on("onClickToolbar", (name:"Right" | "Cross")=>{
            this.modifyCtrl.state.visible = false;
            this.controls.historyCtrl.state.enable = true;
            this.store.selected = this.currSelected
            const initW = this.initValue.w;
            const initH = this.initValue.h;
            const initMtrx = this.initValue.matrix;
            
            if (name == "Cross") { //取消操作
                const compId = this.state.compId
                const imgComp = this.store.compMap[compId];
                imgComp.value.matrix = initMtrx;
                imgComp.value.w = initW;
                imgComp.value.h = initH;
                this.close();

                return;
            }
            //确定操作
            const history = this.controls.historyCtrl.history;
            const compId = this.state.compId
            this.state.compId = "";
            const lastW  = this.lastValue.w;
            const lastH = this.lastValue.h;
            const lastMatrix = this.lastValue.matrix;
            history.record({
                undo: ()=> {
                    const imgComp = this.store.compMap[compId];
                    imgComp.value.matrix = initMtrx;
                    imgComp.value.w = initW;
                    imgComp.value.h = initH;
                },
                redo: ()=> {
                    const imgComp = this.store.compMap[compId];
                    imgComp.value.matrix = lastMatrix;
                    imgComp.value.w = lastW;
                    imgComp.value.h = lastH;
                }
            } as any)

            this.close();
        })

        let tmpM = new Matrix();
        let tmpM2 = new Matrix();

        this.modifyCtrl.on("change", (dtx:number, dty:number)=>{
            const imgComp = this.store.compMap[this.state.compId];

            tmpM.copyFrom(this.modifyCtrl.objContainer.parent.worldTransform);

            // const comp = new CompObject(imgComp);
            // tmpM2.copyFrom(comp.worldTransform)
            // tmpM2.invert()
            // tmpM.prepend(tmpM2)


            this.lastValue.w = this.modifyCtrl.objContainer.width;
            this.lastValue.h = this.modifyCtrl.objContainer.height;
            this.lastValue.matrix= tmpM.getMatrixStr();
            imgComp.value.matrix = this.lastValue.matrix;
            imgComp.value.w = this.lastValue.w;
            imgComp.value.h = this.lastValue.h;
        });
    }

    cancelTransform: ()=>void = ()=>{console.log("not cropper!!")};

    async croppImage(id:string) {

        const imgComp = this.store.compMap[id]
        if (imgComp.compKey != "Image") return;

        this.state.compId = id;
        this.state.visible = true;
        this.controls.historyCtrl.state.enable = false;
        this.modifyCtrl.state.toolbarNames = ["Right", "Cross"];

        const cardBox = this.controls.selectCtrl.getCurrCardBox();
        this.modifyCtrl.baseBox.x = cardBox.left;
        this.modifyCtrl.baseBox.y = cardBox.top;
        this.modifyCtrl.baseBox.w = cardBox.width;
        this.modifyCtrl.baseBox.h = cardBox.height;

        const isChild = this.helper.isStreamCardChild(id)
        if (isChild) {
             await this.croppImageImpl(id, true);
        } else {
            await this.croppImageImpl(id, false);
        }
        const vm = CreateTransform(this.modifyCtrl);

        const editLayer = this.helper.createEditElement(id)
        if (!editLayer) return;

        vm.mount(editLayer.dom);

        this.cancelTransform = ()=>{
            vm.unmount()
            editLayer.destroy();
        }
    }

    async croppImageImpl(id :string, isChild= true) {    
       
       const imgComp = this.store.compMap[id]
        const obj = new CompObject(imgComp);
        const w = obj.width, h = obj.height; //图片的宽高
        const r1 = h / w;
        const imgObj = await createImage(imgComp.value.url)
        const r2 = imgObj.height / imgObj.width;

        this.modifyCtrl.state.baseTop = 0;
        const defaultScale =  imgComp.value.s == undefined || (imgComp.value.s == 1 && imgComp.value.x == 0 && imgComp.value.y == 0);
        let scale = 1;
        let offsetX = 0;
        let offsetY = 0;
        let srcWidth = w;
        let srcHeight = h;
        let srcOffx = 0;
        let srcOffy = 0;

        if (r1 < r2) {
            const s = h / imgObj.height
            srcWidth = imgObj.width * s;
            srcOffx = (w - srcWidth) /2.0;
        }  else {
            //高度不变,计算宽度
            const s = w / imgObj.width
            srcHeight = s * imgObj.height;
            srcOffy = (h - srcHeight) / 2.0;
        }

        if (!defaultScale) {//已经有配置了
            scale = imgComp.value.s;
            offsetX = imgComp.value.x
            offsetY = imgComp.value.y;
        } else {//没有配置 catain 居中处理
            const maxw = Math.max(w, h) / Math.min(w,h);
            scale = maxw
        }   

        let m = new Matrix();
        if (imgComp.value.matrix) {
            m.setMatrixStr(imgComp.value.matrix);
            srcWidth = imgComp.value.w;
            srcHeight = imgComp.value.h;
            this.initValue.matrix = imgComp.value.matrix;
        } else {
            const imgSrc = new ObjsContainer([]);
            imgSrc.rect.width = srcWidth;
            imgSrc.rect.height = srcHeight;
            imgSrc.translate(srcOffx, srcOffy);
            imgSrc.setPivot(4);
            imgSrc.scale(scale, scale)
            imgSrc.translate(offsetX, offsetY)
            m = imgSrc.parent.worldTransform.clone();
            this.initValue.matrix = m.getMatrixStr();
        }

        this.initValue.w = srcWidth;
        this.initValue.h = srcHeight;

        //获取obj的image图片的世界大小
        // const paths = this.helper.getCompTrees(id);
        // const objC =  new ObjsContainer([new CompObject(paths[2])]) //当前对象作为容器
        // const wmx = new Matrix();
        // wmx.copyFrom(objC.parent.worldTransform)
        // wmx.tx = 0;
        // wmx.ty = 0;
        // m.prepend(wmx);
        if (this.controls.selectCtrl.assistMagnet) {
            this.controls.selectCtrl.assistMagnet.enable = false;
        }

        this.modifyCtrl.state.showScaleBridge = false;
        this.modifyCtrl.state.visible = true;
        this.modifyCtrl.objContainer.rect.width = srcWidth;
        this.modifyCtrl.objContainer.rect.height = srcHeight;
        this.modifyCtrl.objContainer.parent.transform.setFromMatrix(m);
        this.modifyCtrl.objContainer.parent.updateTransform();
        this.modifyCtrl.updateState();

        this.currSelected = this.store.selected.slice(0);

        this.store.selected = [];
    }

    close(): void {
        this.cancelTransform();
        this.cancelTransform = ()=>{console.log("empty")};

        const compId = this.state.compId
        const imgComp = this.store.compMap[compId];
        if (imgComp) {
            imgComp.value.matrix = this.initValue.matrix;
            imgComp.value.w = this.initValue.w;
            imgComp.value.h = this.initValue.h;
        }
        this.state.compId = "";
        this.state.visible = false;
    }
}

