import { ModuleRoot } from "queenjs";
import { actions } from "./actions";
import { components } from "./components";
import { stores } from "./stores";
import { BusController } from "@/controllers/natsController";
import { IAppKeys } from "./objects/Application/types";
import { QueentreeApi } from "./objects/Apis";

export class LauncherModule extends ModuleRoot {
  config = this.setConfig({
    supportApps: ["queentree"] as IAppKeys[],
  });

  components = this.useComponents(components);
  store = this.createStore(stores);
  actions = this.createActions(actions);

  queentreeApi = new QueentreeApi(this);

  controls = {
    natsCtrl: new BusController(),
  };

  onReady() {
    this.actions.init();
  }
}

export const { initLauncher, useLauncher } = LauncherModule.hook("Launcher");
