import { queenApi } from "queenjs";
import { ResourceModule } from "..";

export const collectionAction = ResourceModule.action({
  async deleteCollection(record: any) {
    const res = await queenApi.showConfirm({
      content: `删除后无法恢复，确定要删除：${record.title}？`,
      type: "danger",
    });
    if (!res) return;
    await this.https.deleteCollection(record._id);
  },

  async createOrUpdateCollection(record: any) {
    const data = await this.showModal(
      <this.components.CollectionEditModal record={record} />,
      {
        width: "500px",
        maskClosable: false,
        title: record && record._id ? "编辑作品集" : "新增作品集",
      }
    );
    if (record && record._id) {
      await this.https.updateCollection(data);
    } else {
      await this.https.createCollection(data);
    }
  },
  async showCollectionList(record: any) {
    this.showModal(<this.components.CollectionListModal record={record} />, {
      title: `${record.title}作品集`,
      fullscreen: true,
    });
  },
  async collectionDTL(id: string) {
    const res = await this.https.detailCollection(id);
    if (res.errorNo != 200) {
      return;
    }
    return res.result;
  },
  async searchCollection(key: string, commitId: string) {
    try {
      const res = await this.https.searchCollection({ key, commitId });
      return res.result;
    } catch (e) {
      queenApi.messageError("未查询到接收地址数据！");
    }
  },
  async commitCollection(data: any) {
    try {
      const res = await this.https.commitCollection(data);
      return res.result;
    } catch (e) {
      console.log(e);
    }
  },
  async getCommitHistoryDetail(id: string) {
    try {
      const res = await this.https.commitDetail(id);
      return res.result;
    } catch (e) {
      queenApi.messageError("未查询到数据！");
    }
  },
  async updateCommitFlag(data: any) {
    try {
      const res = await this.https.updateCommitFlag(data);
      return res.result;
    } catch (e) {
      console.log(e);
    }
  },
  async updateCommitData(data: any) {
    try {
      const res = await this.https.updateCommitData(data);
      queenApi.messageSuccess("保存成功！");
      return res.result;
    } catch (e) {
      console.log(e);
    }
  },
  async deleteCommitData(record: any) {
    const res = await queenApi.showConfirm({
      content: `删除后无法恢复，确定要删除：${record.title}？`,
      type: "danger",
    });
    if (!res) return;
    await this.https.deleteCommitData(record);
  },
});
