import List from "@/components/AssetsList";
import { useEditor } from "@/modules/editor";
import { EyeOutlined } from "@ant-design/icons-vue";
import { css, cx } from "@linaria/core";
import { IconMore } from "@queenjs/icons";
import { Image, View } from "@queenjs/ui";
import { Divider, Dropdown, Menu, Select } from "ant-design-vue";
import dayjs from "dayjs";
import { defineUI } from "queenjs";
import { defineComponent, onMounted } from "vue";
import { any } from "vue-types";
import { useResource } from "../../";
import { InnerListController } from "./innerListController";
export default defineComponent({
  props: {
    record: any(),
  },
  setup(props, { emit }) {
    const resource = useResource();
    const editor = useEditor();
    const ctrl = InnerListController(resource, editor);

    ctrl.ListCtrl.state.query = { id: props.record._id };
    onMounted(() => {
      ctrl.ListCtrl.loadPage(1);
    });

    return () => {
      return (
        <div>
          <List
            gap="24px"
            class={"p-24px h-full scrollbar"}
            columns={6}
            control={ctrl.ListCtrl}
            item={(item: any) => (
              <ListItem
                item={item}
                record={props.record}
                onMenu={(menu: string, item: any) => {
                  ctrl.onMenuClick(menu, item);
                }}
                onEdit={async (id: string) => {
                  ctrl.onEdit(id);
                }}
              />
            )}
          />
        </div>
      );
    };
  },
});
const ListItem = defineUI({
  props: {
    item: any(),
    record: any(),
  },
  emits: ["edit", "preview", "menu"],
  setup(props, { emit }) {
    return () => {
      const { item, record } = props;
      const h5 = item.h5[0] || {};
      const wks = item.wks || {};
      const options = record?.flags?.map((e: string) => {
        return { label: e, value: e };
      });
      return (
        <div class={cx(itemStyles, "relative")}>
          <View ratio={1} class="overflow-hidden relative">
            <Image
              class="h-1/1 w-1/1 !object-contain bg-[#ebebeb]"
              src={h5?.thumbnail}
            />
            <div class="absolute inset-0 flex items-center justify-center opacity-0 hover:opacity-100 transition-opacity">
              <div
                class="text-white icon_action w-60px leading-60px orange cursor-pointer rounded-1/2 text-center transition-opacity hover:opacity-90 active:opacity-80"
                onClick={() => emit("edit", wks.id)}
              >
                编辑
              </div>
              <div
                class="text-white icon_action w-60px  ml-10px  leading-60px  cursor-pointer rounded-1/2 text-center transition-opacity hover:opacity-90 active:opacity-80"
                onClick={() => emit("menu", "share", wks.id)}
              >
                分享
              </div>
            </div>
            {record.flags && record.flags.length > 0 && (
              <div class={"select_box w-80px"}>
                <Select
                  class={"w-full"}
                  value={wks.flag}
                  options={options}
                  onChange={(v) => {
                    wks.flag = v;
                    emit("menu", "changeFlag", {
                      id: record._id,
                      wkId: wks.id,
                      flag: wks.flag,
                    });
                  }}
                ></Select>
              </div>
            )}
          </View>
          <div class="item_footer rounded-b-4px flex items-center justify-between p-15px">
            <div class="flex-1 w-0">
              <div class="text-white text-bold truncate">{h5?.title}</div>
              <div class="flex items-center text-opacity-60 text-white text-12px mt-5px">
                {dayjs(wks.commitTime).format("YYYY.MM.DD")} 提交
                <Divider type="vertical"></Divider>
                {h5.views || 0} <EyeOutlined class={"ml-6px"} />
              </div>
            </div>
            <Dropdown
              placement="bottom"
              overlay={
                <Menu class="w-90px">
                  <Menu.Item>
                    <div
                      onClick={() =>
                        emit("menu", "delete", {
                          title: h5?.title,
                          worksId:record._id ,
                          id: wks.id,
                        })
                      }
                    >
                      删除
                    </div>
                  </Menu.Item>
                </Menu>
              }
            >
              <IconMore class="ml-10px text-22px cursor-pointer" />
            </Dropdown>
          </div>
        </div>
      );
    };
  },
});
const itemStyles = css`
  .item_footer {
    background: #414141;
  }
  .icon_action {
    background-color: rgba(0, 0, 0, 0.8);
    &.orange {
      background-color: rgba(232, 139, 0, 0.8);
    }
  }
  .select_box {
    position: absolute;
    left: 10px;
    top: 10px;
    .ant-select {
      font-size: 12px;
      .ant-select-selector {
        height: 24px;
        background-color: @inf-component-bg;
        .ant-select-selection-item {
          line-height: 22px;
        }
      }
    }
  }
`;
