import { EditorModule } from "@/modules/editor/module";
import { ResourceModule } from "@/modules/resource";
import { CollectionController } from "@/modules/resource/controllers/CollectionController";
import { PageListController } from "@queenjs/controllers";
import CollectionShare from "./CollectionShare";
import { queenApi } from "queenjs";

export function InnerListController(
  resource: ResourceModule,
  editor: EditorModule
) {
  const ctrl = new CollectionController();

  ctrl.ListCtrl = new PageListController(resource.config?.httpConfig);
  ctrl.ListCtrl.setCrudPrefix("/wk/h5");

  const collectionItemShare = async (id: string) => {
    const recordDtl = await editor.actions.initWkDesign(id);
    editor.actions.switchMode("preview");
    resource.showModal(
      <CollectionShare record={recordDtl} controller={ctrl}>
        {{
          preview: () => (
            <editor.components.Preview class="pointer-events-none" />
          ),
        }}
      </CollectionShare>,
      {
        title: "编辑分享",
        width: "1000px",
        destroyOnClose: true,
      }
    );
  };

  ctrl.onMenuClick = async (key: string, record: any) => {
    if (key == "changeFlag") {
      await resource.actions.updateCommitFlag(record);
      ctrl.ListCtrl.fresh();
    }
    if (key == "share") {
      collectionItemShare(record);
    }
    if (key == "delete") {
      await resource.actions.deleteCommitData(record);
      ctrl.ListCtrl.fresh();
    }
    if (key == "save") {
      await resource.actions.updateCommitData(record);
      ctrl.ListCtrl.fresh();
    }
  };
  ctrl.onEdit = async (id: string) => {
    const _params = new URLSearchParams(decodeURIComponent(location.search));
    const host = _params.get("host");
    const url = `${location.origin}/editor.html?host=${host}#/?id=${id}&isWk=ture`;
    location.href = url;
  };

  return ctrl;
}
