import { ResourceModule } from ".";

export const http = ResourceModule.http({
  // resource
  deleteResource(id: string) {
    return this.request(`/source/delete/${id}`, { method: "POST" });
  },
  publishResource(data: any) {
    return this.request(`sys/source/publish`, { method: "POST", data });
  },
  publishFrame(data: any) {
    return this.request(`sys/frame/publish`, { method: "POST", data });
  },

  createResource(data: createSourceType) {
    return this.request("/source/create", { method: "POST", data });
  },

  updateResouce(data: any) {
    return this.request("/source/update", { method: "POST", data });
  },

  cutVideo(data: { id: string; start: number; end: number }) {
    return this.request("/video/cut", { method: "POST", data });
  },
  queryVideoStatus(id: string) {
    return this.request(`/video/cut/${id}`, { method: "GET" });
  },
  updateVideoUrl(data: { id: string; url: string }) {
    return this.request("source/video/update", { method: "POST", data });
  },

  //   templates
  queryTplsDetail(id: string) {
    return this.request(`/tpls/detail/${id}`, { method: "GET" });
  },

  sourceGen(data) {
    return this.request(`/sourceGen/create`, { method: "POST", data });
  },

  //   promotion
  createPromotion(data: any) {
    return this.request("/h5/create", { method: "POST", data });
  },

  updatePromotion(data: any) {
    return this.request("/h5/update", { method: "POST", data });
  },

  deletePromotion(id: string) {
    return this.request(`/h5/delete/${id}`, { method: "POST" });
  },

  publishPromotion(id: string, publish: boolean) {
    return this.request("/sys/h5/publish", {
      method: "POST",
      data: { _id: id, published: publish },
    });
  },

  //   comp
  createComp(data: any) {
    return this.request("/frame/create", { method: "POST", data });
  },

  updateComp(data: any) {
    return this.request("/frame/update", { method: "POST", data });
  },

  deleteComp(id: string) {
    return this.request(`/frame/delete/${id}`, { method: "POST" });
  },
  //collection
  createCollection(data: any) {
    return this.request("/works/create", { method: "POST", data });
  },

  updateCollection(data: any) {
    return this.request("/works/update", { method: "POST", data });
  },
  deleteCollection(id: string) {
    return this.request(`/works/delete/${id}`, { method: "POST" });
  },
  detailCollection(id: string) {
    return this.request(`/works/detail/${id}`, { method: "GET" });
  },
  searchCollection(params: any) {
    return this.request("/works/search", { method: "GET", params });
  },
  commitCollection(data: any) {
    return this.request("/works/commit", { method: "POST", data });
  },
  commitDetail(id: any) {
    return this.request(`/wk/h5/detail/${id}`, { method: "GET" });
  },
  updateCommitFlag(data: any) {
    return this.request("/wk/update/flag", { method: "POST", data });
  },
  updateCommitData(data: any) {
    return this.request("/wk/h5/update", { method: "POST", data });
  },
  deleteCommitData(data: any) {
    return this.request(`/wk/h5/delete`, { method: "POST", data });
  },
});

/**
 * 接口 说明
 * /sys/frame/list 平台组件列表
 * /sys/source/list?query={"fileType":***} 平台资源列表 fileType：image | video | svg
 * /sys/h5/list 平台模板列表
 * /source/list 用户资源列表
 * /frame/list 用户组件列表
 *
 * /h5/detail/{id}?isSys=true  模板详情
 * 组件和模板detail接口，isSys=true为平台资源
 * isSys字段根据用户profile中是否有roles=["system"]决定
 */
