import { BusController } from "@/controllers/natsController";
import { TreeController } from "@/controllers/queentreeController";
import { Dict_Apis } from "@/dict";
import { PageListController, UploadController } from "@queenjs/controllers";
import { ModuleRoot } from "queenjs";
import { actions } from "./actions";
import { compoents } from "./components";
import { helper } from "./helper";
import { http } from "./http";
import { store } from "./store";

export class ResourceModule extends ModuleRoot {
  config = this.setConfig({
    httpConfig: {
      baseURL: Dict_Apis.promotion,
    },
  });

  actions = this.createActions(actions);
  components = this.useComponents(compoents);
  store = this.createStore([store]);
  https = this.createHttps(http);
  helper = this.createHelper(helper);

  controls = {
    uploader: new UploadController({
      httpConfig: {
        baseURL: Dict_Apis.queentree,
      },
      dir: "queenshow",
    }),
    promotionListCtrl: new PageListController(this.config?.httpConfig),
    materialListCtrl: new PageListController(this.config?.httpConfig),
    matTempListCtrl: new PageListController(this.config?.httpConfig),
    renderTaskListCtrl: new PageListController(this.config?.httpConfig),

    materialImageListCtrl: new PageListController(this.config?.httpConfig),
    materialVideoListCtrl: new PageListController(this.config?.httpConfig),

    // 用户资源
    custImageListCtrl: new PageListController(this.config?.httpConfig),
    custVideoListCtrl: new PageListController(this.config?.httpConfig),
    custCompListCtrl: new PageListController(this.config?.httpConfig),
    custTextListCtrl: new PageListController(this.config?.httpConfig),
    custShapeListCtrl: new PageListController(this.config?.httpConfig),

    // 平台资源
    sysTplListCtrl: new PageListController(this.config?.httpConfig),
    sysImageListCtrl: new PageListController(this.config?.httpConfig),
    sysVideoListCtrl: new PageListController(this.config?.httpConfig),
    sysCompListCtrl: new PageListController<compType, any>(this.config?.httpConfig),
    sysShapeListCtrl: new PageListController<compType, any>(this.config?.httpConfig),
    sysTextListCtrl: new PageListController<compType, any>(this.config?.httpConfig),
  };
  natsBus = new BusController();
  treeController = new TreeController(this.natsBus);

  onReady() {
    this.controls.promotionListCtrl.setCrudPrefix("/h5");
    this.controls.promotionListCtrl.state.size = 12;
    this.controls.materialListCtrl.setCrudPrefix("/source");
    this.controls.materialListCtrl.state.size = 24;
    this.controls.renderTaskListCtrl.setCrudPrefix("/sourceGen");
    this.controls.renderTaskListCtrl.state.size = 24;
    this.controls.matTempListCtrl.setCrudPrefix("/tpls");
    this.controls.matTempListCtrl.state.size = 20;

    this.controls.materialImageListCtrl.setCrudPrefix("/source");
    this.controls.materialImageListCtrl.state.size = 18;
    this.controls.materialImageListCtrl.state.query = { fileType: "image" };

    this.controls.materialVideoListCtrl.setCrudPrefix("/source");
    this.controls.materialVideoListCtrl.state.size = 18;
    this.controls.materialVideoListCtrl.state.query = { fileType: "video" };

    this.controls.custImageListCtrl.setCrudPrefix("/source");
    this.controls.custImageListCtrl.state.size = 20;
    this.controls.custImageListCtrl.state.query = { fileType: "image" };

    this.controls.custVideoListCtrl.setCrudPrefix("/source");
    this.controls.custVideoListCtrl.state.size = 20;
    this.controls.custVideoListCtrl.state.query = { fileType: "video" };

    this.controls.custCompListCtrl.setCrudPrefix("/frame");
    this.controls.custCompListCtrl.state.size = 20;
    this.controls.custCompListCtrl.state.query = { type: "comp"};

    this.controls.custTextListCtrl.setCrudPrefix("/frame");
    this.controls.custTextListCtrl.state.size = 20;
    this.controls.custTextListCtrl.state.query = { type: "text"};
    
    this.controls.custShapeListCtrl.setCrudPrefix("/frame");
    this.controls.custShapeListCtrl.state.size = 20;
    this.controls.custShapeListCtrl.state.query = { type: "shape"};

    this.controls.sysTplListCtrl.setCrudPrefix("/sys/h5");
    this.controls.sysTplListCtrl.state.size = 20;
    this.controls.sysTplListCtrl.state.query = { published: true};

    this.controls.sysImageListCtrl.setCrudPrefix("/sys/source");
    this.controls.sysImageListCtrl.state.size = 20;
    this.controls.sysImageListCtrl.state.query = { fileType: "image" , isSvg: {$ne: true}, published: true};

    this.controls.sysVideoListCtrl.setCrudPrefix("/sys/source");
    this.controls.sysVideoListCtrl.state.size = 20;
    this.controls.sysVideoListCtrl.state.query = { fileType: "video" , published: true};

    this.controls.sysCompListCtrl.setCrudPrefix("sys/frame");
    this.controls.sysCompListCtrl.state.size = 20;
    this.controls.sysCompListCtrl.state.query = { type: "comp", published: true};
   
    this.controls.sysShapeListCtrl.setCrudPrefix("/sys/frame");
    this.controls.sysShapeListCtrl.state.size = 20;
    this.controls.sysShapeListCtrl.state.query = { type: "shape", published: true};
   
    this.controls.sysTextListCtrl.setCrudPrefix("/sys/frame");
    this.controls.sysTextListCtrl.state.size = 20;
    this.controls.sysTextListCtrl.state.query = { type: "text", published: true};

    this.natsBus.init();
  }
}

export const { useResource, initResource } = ResourceModule.hook("Resource");
