import * as echarts from "echarts";
import { defineComponent, onMounted, ref, watch } from "vue";
import { array } from "vue-types";

export default defineComponent({
  props: {
    data: array().isRequired,
  },
  setup(props) {
    const echartRef = ref();
    let chartInstance: any;
    let mapLoaded = false;

    const option = {
      tooltip: {
        trigger: "item",
        showDelay: 0,
        transitionDuration: 0.2,
      },
      visualMap: {
        show: false,
        inRange: {
          color: [
            "#313695",
            "#4575b4",
            "#74add1",
            "#abd9e9",
            "#e0f3f8",
            "#ffffbf",
            "#fee090",
            "#fdae61",
            "#f46d43",
            "#d73027",
            "#a50026",
          ],
        },
      },
      series: [
        {
          name: "统计分析",
          type: "map",
          roam: true,
          map: "china",
          boundingCoords: [
            // 定位左上角经纬度
            [68.99814, 50.74432],
            // 定位右下角经纬度
            [139.93812, 25.23403],
          ],
          emphasis: {
            label: {
              show: true,
            },
          },
          data: [] as any[],
        },
      ],
    };

    onMounted(() => {
      initEchart(echartRef.value);
    });

    watch(
      () => props.data,
      () => {
        if (mapLoaded) {
          updateEchart();
        }
      }
    );

    function updateEchart() {
      option.series[0].data = props.data;
      chartInstance.setOption(option);
    }

    function initEchart(el: HTMLElement) {
      chartInstance = echarts.init(el);
      chartInstance.showLoading();

      fetch(
        "https://geo.datav.aliyun.com/areas_v3/bound/100000_full.json"
      ).then(async (res) => {
        const json = await res.json();
        chartInstance.hideLoading();

        echarts.registerMap("china", json);
        updateEchart();
        mapLoaded = true;
      });
    }

    return () => {
      return <div class="h-300px" ref={echartRef}></div>;
    };
  },
});
