import { ResourceModule } from "@/modules/resource";
import { CollectionController } from "@/modules/resource/controllers/CollectionController";
import { PageListController } from "@queenjs/controllers";

export function createCollectionController(resource: ResourceModule) {
  const ctrl = new CollectionController();

  ctrl.ListCtrl = new PageListController(resource.config?.httpConfig);
  ctrl.ListCtrl.setCrudPrefix("/works");

  ctrl.createCollection = async () => {
    await resource.actions.createOrUpdateCollection(null);
    ctrl.ListCtrl.fresh();
  };
  ctrl.onPreview = (record: any) => {
    resource.actions.showCollectionList(record);
  };
  ctrl.onMenuClick = async (name: string, record: any) => {
    if (name == "delete") {
      await resource.actions.deleteCollection(record);
      ctrl.ListCtrl.fresh();
    } else if (name == "update") {
      if (!record._id) {
        return;
      }
      const detail = await resource.actions.collectionDTL(record._id);
      await resource.actions.createOrUpdateCollection(detail);
      ctrl.ListCtrl.fresh();
    }
  };
  ctrl.onEdit = async (record: any) => {
    if (!record._id) {
      return;
    }
    await resource.https.updateCollection(record);
    ctrl.ListCtrl.fresh();
  };
  return ctrl;
}
